package visitors;

import nodetypes.Expression;
import nodetypes.Value;
import nodetypes.binary.Addition;
import nodetypes.binary.BinaryOperation;
import nodetypes.binary.Division;
import nodetypes.binary.Multiplication;
import nodetypes.binary.Subtraction;
import nodetypes.unary.Negative;
import nodetypes.unary.SquareRoot;

public class PrintVisitor implements Visitor {
	private final String openBracket = "(";
	private final String closeBracket = ")";
	@Override
	public int visit(Expression exp) {
		exp.accept(this);
		return 0;
	}

	private void show(BinaryOperation bo, Character ch) {
		System.out.print(openBracket);
		bo.getLeft().accept(this);
		System.out.print(ch);
		bo.getRight().accept(this);
		System.out.print(closeBracket);
	}
	
	@Override
	public int visit(Addition a) {
		show(a, '+');
		return 0;
	}

	@Override
	public int visit(Subtraction s) {
		show(s, '-');
		return 0;
	}

	@Override
	public int visit(Multiplication m) {
		show(m, '*');
		return 0;
	}

	@Override
	public int visit(Division d) {
		show(d, '/');
		return 0;
	}

	@Override
	public int visit(Value v) {
		System.out.print(v.getValue());
		return 0;
	}

	@Override
	public int visit(Negative n) {
		System.out.print(openBracket);
		System.out.print("-");
		n.getSubExpression().accept(this);
		System.out.print(closeBracket);
		return 0;
	}

	@Override
	public int visit(SquareRoot s) {
		System.out.print(openBracket);
		System.out.print("v");
		s.getSubExpression().accept(this);
		System.out.print(closeBracket);
		return 0;
	}	
}
