package visitors;

import nodetypes.Expression;
import nodetypes.Value;
import nodetypes.binary.Addition;
import nodetypes.binary.BinaryOperation;
import nodetypes.binary.Division;
import nodetypes.binary.Multiplication;
import nodetypes.binary.Subtraction;
import nodetypes.unary.Negative;
import nodetypes.unary.SquareRoot;
import nodetypes.unary.UnaryOperation;

public class StepByStepComputeVisitor implements Visitor{
	private Visitor v = new ComputeVisitor();
	
	public int visit(Expression exp) {
		return exp.accept(this);
	}

	public int check(BinaryOperation bo) {
		int left = bo.getLeft().accept(this);
		if(2 == left) {
			Expression l = new Value(v.visit(bo.getLeft()));
			bo.setLeft(l);
		}
		
		int right = bo.getRight().accept(this);
		if(2 == right) {
			Expression r = new Value(v.visit(bo.getRight()));		
			bo.setRight(r);
		}
		return left + right;
	}
	
	
	public int check(UnaryOperation uo) {
		int r = uo.getSubExpression().accept(this);
		if(2 == r) {
			Expression exp = new Value(v.visit(uo.getSubExpression()));		
			uo.setSubExpression(exp);
		}
		
		if(r == 1) {
			return r + 1;
		} else {
			return r + 2;//do not know why, but it works :))
		}
	}
	
	public int visit(Addition a) {
		return check(a);
	}

	@Override
	public int visit(Subtraction s) {
		return check(s);
	}

	@Override
	public int visit(Multiplication m) {
		return check(m);
	}

	@Override
	public int visit(Division d) {
		return check(d);
	}

	@Override
	public int visit(Negative n) {
		return check(n);
	}

	@Override
	public int visit(SquareRoot s) {
		return check(s);
	}
	
	@Override
	public int visit(Value v) {
		return 1;
	}
}
